from plotly.offline import plot
import plotly.graph_objs as go
import pandas as pd
import talib as ta
from matplotlib.widgets import Cursor
import matplotlib.pyplot as plt

def plot_candlestick_from_csv(csv_file_path):
	df = pd.read_csv(csv_file_path)
	plot_candlestick(df)

'''
We can validate our results by plotting the candles and visually check 
against the patterns found. This visualizes the data using Plotly. 
The dataset and the plot can be compared side by side and the patterns 
can be validated easily by matching the indexes.
Before calling this, we should have extracted all the candlestick patterns 
using TA-Lib. We should have ranked them based on the 
“Overall performance rank” and selected the best performance pattern 
for each candle.
'''
def plot_candlestick(df, symbol_name="", plot_title=""):
	o = df['Open'].astype(float)
	h = df['High'].astype(float)
	l = df['Low'].astype(float)
	c = df['Close'].astype(float)
	p = df['candlestick_pattern'].astype(str)

	trace = go.Candlestick(
	            open=o,
	            high=h,
	            low=l,
	            close=c,
	            text=p)
	data = [trace]

	layout = {
	    'title': plot_title,
	    'yaxis': {'title': 'Price'},
	    'xaxis': {'title': 'Index Number'},

	}
	fig = dict(data=data, layout=layout)
	plot(fig, filename= symbol_name +'_candles.html')

def plot_technical_indicators(df):
	# plot with various axes scales
	plt.figure(1)
	# plt.subplot(221)
	plot_history(df,'Close', 'Turnover')

	plt.figure(2)	
	# plt.subplot(222)
	plot_rsi(df)

	plt.figure(3)
	# plt.subplot(223)
	plot_sma(df)
	
	plt.figure(4)
	# plt.subplot(224)
	plot_ema(df)

	# plt.figure()
	# plt.subplot(225)
	# plot_adx(df)

	# plt.figure()
	# plt.subplot(226)
	# plot_bbands(df)

	# plt.figure()
	# plt.subplot(227)
	# plot_sstochastic(df)

	# plt.figure()
	# plt.subplot(228)
	# plot_fstochastic(df)

	# Adjust the subplot layout
	# plt.subplots_adjust(top=0.92, bottom=0.08, left=0.10, right=0.95, hspace=0.25,
	#                     wspace=0.5)
	return plt

def plot_history(df, plot_points=['Close'], secondary_y="Turnover"):
	df[plot_points].plot(secondary_y=secondary_y)
	plt.title('Price History')
	plt.grid(True)
	return plt

'''
The relative strength index is a technical indicator used in the 
analysis of financial markets. It is intended to chart the current 
and historical strength or weakness of a stock or market based on 
the closing prices of a recent trading period.
'''
def plot_rsi(df):
	df['RSI'] = ta.RSI(df['Close'],14)
	df['RSI'].plot()
	plt.title('RSI(14)')
	plt.grid(True)
	return plt

def plot_mom(df):
	df['MOM'] = ta.MOM(df['Close'],10)
	df['MOM'].plot()
	plt.title('Momentum - MOM(10)')
	plt.grid(True)
	return plt

def plot_dmi(df):
	df['DMI'] = ta.DX(df['High'],df['Low'],df['Close'],timeperiod=14)
	df['DMI'].plot()
	plt.title('Directional Movement Index - DMI(14)')
	plt.grid(True)
	return plt

def plot_macd(df):
	df['macd'], df['macdsignal'], df['macdhist'] = ta.MACDEXT(df['Close'], fastperiod=12, fastmatype=0, slowperiod=26, slowmatype=0, signalperiod=9, signalmatype=0)
	df[['macd','macdsignal', 'macdhist']].plot()
	plt.title('MACD(12, 26)')
	plt.grid(True)
	return plt

'''
Simple moving average (SMA) calculates the average of a selected 
range of closing prices, by the number of periods in that range.
'''
def plot_sma(df):
	df['SMA(10)'] = ta.SMA(df['Close'],10)
	df['SMA(50)'] = ta.SMA(df['Close'],50)
	df[['Close','SMA(10)', 'SMA(50)']].plot()
	plt.title('SMA(10) & SMA(50)')
	plt.grid(True)
	return plt

'''
(EMA) is a type of moving average (MA) that places a greater weight 
and significance on the most recent data points. That is it is 
generally known as Exponentially Weighted Moving Average.
'''
def plot_ema(df):
	df['EMA(10)'] = ta.EMA(df['Close'], timeperiod = 10)
	df[['Close','EMA(10)']].plot()
	plt.title('EMA(10)')
	plt.grid(True)
	return plt

'''
Average Directional Movement Index(Momentum Indicator) - ADX can be 
used to help measure the overall strength of a trend. The ADX 
indicator is an average of expanding price range values.
'''
def plot_adx(df):
	df['ADX'] = ta.ADX(df['High'],df['Low'], df['Close'], timeperiod=20)
	df[['ADX']].plot()
	plt.title('ADX(20)')
	plt.grid(True)
	return plt

'''
Bollinger Bands are a type of statistical chart characterizing the 
prices and volatility over time of a financial instrument or commodity, 
using a formulaic method propounded by John Bollinger.
'''
def plot_bbands(df):
	df['BBands-U'], df['BBands-M'], df['BBands-L'] = ta.BBANDS(df['Close'], timeperiod =20)
	df[['Close','BBands-U','BBands-M','BBands-L']].plot()
	plt.title('Bollinger Bands(20)')
	plt.grid(True)
	return plt

def plot_sstochastic(df):
	df['SStochastic(14)'], df['SStochastic(3)'] = ta.STOCH(df['High'], df['Low'], df['Close'], fastk_period=14, slowk_period=3, slowk_matype=0, slowd_period=3, slowd_matype=0)
	df[['SStochastic(14)','SStochastic(3)']].plot()
	plt.title('Slow Stochastic(k=14, d=3)')
	plt.grid(True)
	return plt

def plot_fstochastic(df):
	df['FStochastic(14)'], df['FStochastic(3)'] = ta.STOCHF(df['High'], df['Low'], df['Close'], fastk_period=14, fastd_period=3, fastd_matype=0)
	df[['FStochastic(14)','FStochastic(3)']].plot()
	plt.title('Fast Stochastic(k=14, d=3)')
	plt.grid(True)
	return plt




from fastquant import backtest
from fbprophet import Prophet
from fbprophet.plot import add_changepoints_to_plot

from matplotlib import pyplot as plt

import click


def backtest_smac_strategy(df, fast_period=10, slow_period=50):
	result = backtest('smac', df, fast_period=fast_period, slow_period=slow_period, verbose=False)
	print(result[['fast_period', 'slow_period', 'init_cash', 'final_value', 'pnl']].head())

def backtest_emac_strategy(df, fast_period=10, slow_period=50):
	result = backtest('emac', df, fast_period=fast_period, slow_period=slow_period, verbose=False)
	print(result[['fast_period', 'slow_period', 'init_cash', 'final_value', 'pnl']].head())

def backtest_rsi_strategy(df, rsi_period=14, rsi_lower=30):
	result = backtest('rsi', df, rsi_period=rsi_period, rsi_upper=70, rsi_lower=rsi_lower, verbose=False)
	print(result[['rsi_period', 'rsi_upper', 'rsi_lower', 'init_cash', 'final_value', 'pnl']].head())

def backtest_macd_strategy(df, fast_period=12, slow_period=26):
	result = backtest('macd', df, fast_period=fast_period, slow_period=slow_period, signal_period=9, 
		sma_period=30, dir_period=10, verbose=False)
	print(result[['fast_period', 'slow_period', 'signal_period', 'init_cash', 'final_value', 'pnl']].head())

def backtest_bbands_strategy(df, period=20, devfactor=2.0):
	result = backtest('bbands', df, period=period, devfactor=devfactor, verbose=False)
	print(result[['period', 'devfactor', 'init_cash', 'final_value', 'pnl']].head())

def backtest_multi_strategy(df, key_variable="smac", fast_period=10, slow_period=50, rsi_lower=30, rsi_upper=70):
	strats = { 
		key_variable: {"fast_period": fast_period, "slow_period": slow_period},
		"rsi": {"rsi_lower": rsi_lower, "rsi_upper": rsi_upper} 
	}
	result = backtest("multi", df, strats=strats, verbose=False)
	# print(result[[key_variable +'.fast_period', key_variable+'.slow_period', 'rsi.rsi_lower', 'rsi.rsi_upper', 'init_cash', 'final_value', 'pnl']].head())
	return result

# def backtest_custom_strategy(df, symbol, strategy, upper_limit, lower_limit):
# 	plt, result = daily_forecast(df, symbol, strategy, upper_limit, lower_limit, 0)
# 	return result

'''
This powerful strategy allows to backtest our own trading strategies 
using any type of model after the forecast!
Predictions based on any model can be used as a custom indicator to 
be backtested using fastquant. You just need to add a custom column 
in the input dataframe, and set values for upper_limit and lower_limit.
The strategy is structured similar to RSIStrategy where you can set an 
upper_limit, above which the asset is sold (considered "overbought"), 
and a lower_limit, below which the asset is bought (considered "underbought). 
upper_limit is set to 95 by default, while lower_limit is set to 5 by default.

We are going to use the custom strategy to backtest a custom indicator based 
on in-sample time series forecasts. The forecasts are generated using 
Facebook's Prophet package.
'''
def daily_forecast(df, symbol, strategy, upper_limit=1.5, lower_limit=1.5, periods=0):
	# Fit model on closing prices
	ts = df.reset_index()[["dt", "close"]]
	ts.columns = ['ds', 'y']
	if not ts['y'].count() >= 2:
		click.secho("Dataframe has less than 2 non-NaN rows. Cannot fit the model.", fg='red', nl=True)
		return
	# m = Prophet(daily_seasonality=True, yearly_seasonality=True).fit(ts)
	m = Prophet(growth="linear",
            seasonality_mode='additive',
            daily_seasonality=False,
            weekly_seasonality=True,
            yearly_seasonality=True,
            interval_width=0.95, #uncertainty
            holidays=None,
            n_changepoints=20,
           ) 
	m.add_country_holidays(country_name='IN')
	m.fit(ts)

	future = m.make_future_dataframe(periods=periods, freq='D')
	future.tail()

	# Predict and plot
	forecast = m.predict(future)
	forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']].tail()

	if periods > 0:
		fig1 = m.plot(forecast, uncertainty=True)
		# fig2 = m.plot_components(forecast)
		a = add_changepoints_to_plot(fig1.gca(), m, forecast)
		# fig1.axes[0].set_xlim(datestring_to_datetime("2020-02-01"),
		#                       datestring_to_datetime("2020-04-30")
		#                      )
		fig1.axes[0].set_xlabel('Date')
		fig1.axes[0].set_ylabel('Price')
		plt.subplot()
		plt.title(symbol.upper()+' Forecasted Closing Price, trend and strategic points - Strategy:' + strategy.upper(), fontsize=15)
		plt.subplots_adjust(top=0.92, bottom=0.08, left=0.10, right=0.90, hspace=0.25,
		                    wspace=0.5)

	# Convert predictions to expected 1 day returns
	expected_1day_return = forecast.set_index("ds").yhat.pct_change().shift(-1).multiply(100)

	# Backtest the predictions, given that we buy the given symbol when the predicted 
	# next day return is > +1.5%, and sell when it's < -1.5%.
	df[strategy] = expected_1day_return.multiply(-1)
	if strategy == 'custom':
		result = backtest(strategy, df.dropna(),upper_limit=upper_limit, lower_limit=-lower_limit)
	else:
		result = backtest(strategy, df.dropna())
	print(result[['init_cash', 'final_value', 'pnl']].head())
	return plt, result



import click
import pandas as pd 

from nseta.live.live import get_quote
from nseta.cli.inputs import *
from datetime import datetime

QUOTE_KEYS = ['companyName', 'lastPrice', 'pChange', 'open', 'dayHigh', 'dayLow', 'closePrice', 'previousClose', 'high52', 'low52']
VOLUME_KEYS = ['quantityTraded', 'totalTradedVolume', 'totalTradedValue']

@click.command(help='Get live price quote of a security')
@click.option('--symbol', '-S',  help='Security code')
@click.option('--series', default='EQ', help='Default series - EQ (Equity)')
def live_quote(symbol, series):
    if not validate_symbol(symbol):
        print_help_msg(live_quote)
        return
    result = get_quote(symbol)
    if len(result['data']) == 0:
        click.secho("Please check the inputs. Could not fetch the data.", fg='red', nl=True)
        return
    data = result['data'][0]
    time = result['lastUpdateTime']
    quote_data = []
    for key in QUOTE_KEYS:
        quote_data.append(data[key])
    quote_data = [quote_data]

    volume_data = []
    for key in VOLUME_KEYS:
        volume_data.append(data[key])
    volume_data = [volume_data]

    pipeline_data = []
    for x in range(1,5):
        buy_qty_key = 'buyQuantity' + str(x)
        buy_prc_key = 'buyPrice' + str(x)
        sell_qty_key = 'sellQuantity' + str(x)
        sell_prc_key = 'sellPrice' + str(x)
        columns = [buy_qty_key, buy_prc_key, sell_qty_key, sell_prc_key]
        row = []
        for column in columns:
            row.append(data[column])
        pipeline_data.append(row)

    dfquote = pd.DataFrame(quote_data, columns = ['Name', 'Last_Trade_Price', 'Price_Change', 
        'Open', 'High', 'Low', 'Close', 'Prev_Close', '52wk_High', '52wk_Low'])
    dfquote.set_index('Name', inplace=True)
    dfvolume = pd.DataFrame(volume_data, columns = ['Quantity_Traded','Total_Traded_Volume', 'Total_Traded_Value'])
    dfvolume.set_index('Quantity_Traded', inplace=True) 
    dfpipeline = pd.DataFrame(pipeline_data, columns = ['Bid_Quantity','Bid_Price', 'Offer_Quantity', 'Offer_Price'])
    dfpipeline.set_index('Bid_Quantity', inplace=True) 
    print ('As of ' + time + '\n')
    print (dfquote.head())
    print ('\n')
    print (dfvolume.head())
    print ('\n')
    print (dfpipeline.head())
    print ('\n')
    


# https://github.com/CanerIrfanoglu/medium
# Note - 1
# Only some patterns have bull and bear versions. 
# However, to make the process unified and for codability purposes 
# all patterns are labeled with "_Bull" and "_Bear" tags.
# Both versions of the single patterns are given same performance rank, 
# since they will always return only 1 version.  

# Note - 2 
# Following TA-Lib patterns are excluded from the analysis since 
# the corresponding ranking not found:
# CounterAttack, Longline, Shortline, Stalledpattern, Kickingbylength
# http://thepatternsite.com/rank.html

candle_rankings = {
        "CDL3LINESTRIKE_Bull": 1,
        "CDL3LINESTRIKE_Bear": 2,
        "CDL3BLACKCROWS_Bull": 3,
        "CDL3BLACKCROWS_Bear": 3,
        "CDLEVENINGSTAR_Bull": 4,
        "CDLEVENINGSTAR_Bear": 4,
        "CDLTASUKIGAP_Bull": 5,
        "CDLTASUKIGAP_Bear": 5,
        "CDLINVERTEDHAMMER_Bull": 6,
        "CDLINVERTEDHAMMER_Bear": 6,
        "CDLMATCHINGLOW_Bull": 7,
        "CDLMATCHINGLOW_Bear": 7,
        "CDLABANDONEDBABY_Bull": 8,
        "CDLABANDONEDBABY_Bear": 8,
        "CDLBREAKAWAY_Bull": 10,
        "CDLBREAKAWAY_Bear": 10,
        "CDLMORNINGSTAR_Bull": 12,
        "CDLMORNINGSTAR_Bear": 12,
        "CDLPIERCING_Bull": 13,
        "CDLPIERCING_Bear": 13,
        "CDLSTICKSANDWICH_Bull": 14,
        "CDLSTICKSANDWICH_Bear": 14,
        "CDLTHRUSTING_Bull": 15,
        "CDLTHRUSTING_Bear": 15,
        "CDLINNECK_Bull": 17,
        "CDLINNECK_Bear": 17,
        "CDL3INSIDE_Bull": 20,
        "CDL3INSIDE_Bear": 56,
        "CDLHOMINGPIGEON_Bull": 21,
        "CDLHOMINGPIGEON_Bear": 21,
        "CDLDARKCLOUDCOVER_Bull": 22,
        "CDLDARKCLOUDCOVER_Bear": 22,
        "CDLIDENTICAL3CROWS_Bull": 24,
        "CDLIDENTICAL3CROWS_Bear": 24,
        "CDLMORNINGDOJISTAR_Bull": 25,
        "CDLMORNINGDOJISTAR_Bear": 25,
        "CDLXSIDEGAP3METHODS_Bull": 27,
        "CDLXSIDEGAP3METHODS_Bear": 26,
        "CDLTRISTAR_Bull": 28,
        "CDLTRISTAR_Bear": 76,
        "CDLGAPSIDESIDEWHITE_Bull": 46,
        "CDLGAPSIDESIDEWHITE_Bear": 29,
        "CDLEVENINGDOJISTAR_Bull": 30,
        "CDLEVENINGDOJISTAR_Bear": 30,
        "CDL3WHITESOLDIERS_Bull": 32,
        "CDL3WHITESOLDIERS_Bear": 32,
        "CDLONNECK_Bull": 33,
        "CDLONNECK_Bear": 33,
        "CDL3OUTSIDE_Bull": 34,
        "CDL3OUTSIDE_Bear": 39,
        "CDLRICKSHAWMAN_Bull": 35,
        "CDLRICKSHAWMAN_Bear": 35,
        "CDLSEPARATINGLINES_Bull": 36,
        "CDLSEPARATINGLINES_Bear": 40,
        "CDLLONGLEGGEDDOJI_Bull": 37,
        "CDLLONGLEGGEDDOJI_Bear": 37,
        "CDLHARAMI_Bull": 38,
        "CDLHARAMI_Bear": 72,
        "CDLLADDERBOTTOM_Bull": 41,
        "CDLLADDERBOTTOM_Bear": 41,
        "CDLCLOSINGMARUBOZU_Bull": 70,
        "CDLCLOSINGMARUBOZU_Bear": 43,
        "CDLTAKURI_Bull": 47,
        "CDLTAKURI_Bear": 47,
        "CDLDOJISTAR_Bull": 49,
        "CDLDOJISTAR_Bear": 51,
        "CDLHARAMICROSS_Bull": 50,
        "CDLHARAMICROSS_Bear": 80,
        "CDLADVANCEBLOCK_Bull": 54,
        "CDLADVANCEBLOCK_Bear": 54,
        "CDLSHOOTINGSTAR_Bull": 55,
        "CDLSHOOTINGSTAR_Bear": 55,
        "CDLMARUBOZU_Bull": 71,
        "CDLMARUBOZU_Bear": 57,
        "CDLUNIQUE3RIVER_Bull": 60,
        "CDLUNIQUE3RIVER_Bear": 60,
        "CDL2CROWS_Bull": 61,
        "CDL2CROWS_Bear": 61,
        "CDLBELTHOLD_Bull": 62,
        "CDLBELTHOLD_Bear": 63,
        "CDLHAMMER_Bull": 65,
        "CDLHAMMER_Bear": 65,
        "CDLHIGHWAVE_Bull": 67,
        "CDLHIGHWAVE_Bear": 67,
        "CDLSPINNINGTOP_Bull": 69,
        "CDLSPINNINGTOP_Bear": 73,
        "CDLUPSIDEGAP2CROWS_Bull": 74,
        "CDLUPSIDEGAP2CROWS_Bear": 74,
        "CDLGRAVESTONEDOJI_Bull": 77,
        "CDLGRAVESTONEDOJI_Bear": 77,
        "CDLHIKKAKEMOD_Bull": 82,
        "CDLHIKKAKEMOD_Bear": 81,
        "CDLHIKKAKE_Bull": 85,
        "CDLHIKKAKE_Bear": 83,
        "CDLENGULFING_Bull": 84,
        "CDLENGULFING_Bear": 91,
        "CDLMATHOLD_Bull": 86,
        "CDLMATHOLD_Bear": 86,
        "CDLHANGINGMAN_Bull": 87,
        "CDLHANGINGMAN_Bear": 87,
        "CDLRISEFALL3METHODS_Bull": 94,
        "CDLRISEFALL3METHODS_Bear": 89,
        "CDLKICKING_Bull": 96,
        "CDLKICKING_Bear": 102,
        "CDLDRAGONFLYDOJI_Bull": 98,
        "CDLDRAGONFLYDOJI_Bear": 98,
        "CDLCONCEALBABYSWALL_Bull": 101,
        "CDLCONCEALBABYSWALL_Bear": 101,
        "CDL3STARSINSOUTH_Bull": 103,
        "CDL3STARSINSOUTH_Bear": 103,
        "CDLDOJI_Bull": 104,
        "CDLDOJI_Bear": 104
    }



import numpy as np
from nseta.analytics.candle_rankings import candle_rankings
import talib
from itertools import compress

# patterns not found in the patternsite.com
exclude_items = ('CDLCOUNTERATTACK',
                 'CDLLONGLINE',
                 'CDLSHORTLINE',
                 'CDLSTALLEDPATTERN',
                 'CDLKICKINGBYLENGTH')

def get_candle_funcs():
	funcs = talib.get_function_groups()['Pattern Recognition'] 
	candle_names = [candle for candle in funcs if candle not in exclude_items]

	return candle_names

'''
TA-Lib creates individual columns for each pattern. While 0 corresponds 
to no pattern, positive values represent bullish patterns and negative 
values represent bearish patterns.
'''
def create_pattern_data(data_frame):
	df = data_frame
	# extract OHLC
	O = df['Open'].astype(float)
	H = df['High'].astype(float)
	L = df['Low'].astype(float)
	C = df['Close'].astype(float)

	candle_names = get_candle_funcs()

	# create columns for each pattern
	for candle in candle_names:
		# below is same as;
		# df["CDL3LINESTRIKE"] = talib.CDL3LINESTRIKE(O, H, L, C)
		df[candle] = getattr(talib, candle)(O, H, L, C)

	return df

'''
Creates the pattern labels and found pattern counts.
We basically have 3 cases:
	No Pattern: Fill the cell with “NO_PATTERN”
	Single Pattern: Fill the cell with Pattern Name
	Multiple Patterns: Fill the cell with lowest (best) ranking Pattern Name
'''
def pick_best_rank_from_pattern(data_frame):
	df = data_frame
	df['candlestick_pattern'] = np.nan
	df['candlestick_match_count'] = np.nan
	candle_names = get_candle_funcs()

	for index, row in df.iterrows():

		# no pattern found
		if len(row[candle_names]) - sum(row[candle_names] == 0) == 0:
			df.loc[index,'candlestick_pattern'] = "NO_PATTERN"
			df.loc[index, 'candlestick_match_count'] = 0
		# single pattern found
		elif len(row[candle_names]) - sum(row[candle_names] == 0) == 1:
			# bull pattern 100 or 200
			if any(row[candle_names].values > 0):
				pattern = list(compress(row[candle_names].keys(), row[candle_names].values != 0))[0] + '_Bull'
				df.loc[index, 'candlestick_pattern'] = pattern
				df.loc[index, 'candlestick_match_count'] = 1
			# bear pattern -100 or -200
			else:
				pattern = list(compress(row[candle_names].keys(), row[candle_names].values != 0))[0] + '_Bear'
				df.loc[index, 'candlestick_pattern'] = pattern
				df.loc[index, 'candlestick_match_count'] = 1
		# multiple patterns matched -- select best performance
		else:
			# filter out pattern names from bool list of values
			patterns = list(compress(row[candle_names].keys(), row[candle_names].values != 0))
			container = []
			for pattern in patterns:
				if row[pattern] > 0:
					container.append(pattern + '_Bull')
				else:
					container.append(pattern + '_Bear')
				rank_list = [candle_rankings[p] for p in container]
				if len(rank_list) == len(container):
					rank_index_best = rank_list.index(min(rank_list))
					df.loc[index, 'candlestick_pattern'] = container[rank_index_best]
					df.loc[index, 'candlestick_match_count'] = len(container)

	return df

'''
We will extract candlestick patterns using TA-Lib (supports 61 patterns as of Sep 2020). 
We will rank them based on the “Overall performance rank” and select the best performance 
pattern for each candle.
'''
def recognize_candlestick_pattern(data_frame, steps):
	"""
	Recognizes candlestick patterns and appends 2 additional columns to df;
	1st - Best Performance candlestick pattern matched by www.thepatternsite.com
	2nd - # of matched patterns
	"""
	df = create_pattern_data(data_frame)
	candle_names = get_candle_funcs()
	df = pick_best_rank_from_pattern(df)

	# clean up candle columns
	if not steps:
		cols_to_drop = candle_names
		df.drop(cols_to_drop, axis = 1, inplace = True)

	return df

def model_candlestick(df, steps):
	return recognize_candlestick_pattern(df, steps)




from nseta.analytics.model import *
from nseta.common.history import get_history
from nseta.plots.plots import *
from nseta.cli.inputs import *

import click
from datetime import datetime

@click.command(help='Create candlestick model.Plot uncovered patterns')
@click.option('--symbol', '-S',  help='Security code')
@click.option('--start', '-s', help='Start date in yyyy-mm-dd format')
@click.option('--end', '-e', help='End date in yyyy-mm-dd format')
@click.option('--file', '-o', 'file_name',  help='Output file name. Default is {symbol}.csv')
@click.option('--steps/--no-steps', default=False, help='--steps for saving intermediate steps in output file')
@click.option('--format', '-f', default='csv',  type=click.Choice(['csv', 'pkl']),
                help='Output format, pkl - to save as Pickel and csv - to save as csv')
def create_cdl_model(symbol, start, end, file_name, steps, format):
	if not validate_inputs(start, end, symbol):
		print_help_msg(create_cdl_model)
		return
	sd = datetime.strptime(start, "%Y-%m-%d").date()
	ed = datetime.strptime(end, "%Y-%m-%d").date()

	df = get_history(symbol, sd, ed)
	df.set_index('Date', inplace=True)
	df = model_candlestick(df, steps)
	click.echo(df.head())
	if not file_name:
		file_name = symbol + '.' + format
	if format == 'csv':
		df.to_csv(file_name)
	else:
		df.to_pickle(file_name)
	click.secho('Model saved to: {}'.format(file_name), fg='green', nl=True)
	click.secho('Candlestick pattern model plot saved to: {}'.format(symbol +'_candles.html'), fg='green', nl=True)	
	plot_candlestick(df, symbol, 'Candlestick Pattern Model Recognition for ' + symbol)



import click
from datetime import datetime

def validate_inputs(start, end,symbol):
    try:
        sd = datetime.strptime(start, "%Y-%m-%d").date()
        ed = datetime.strptime(end, "%Y-%m-%d").date()
    except:
        click.secho("Please provide start and end date in format yyyy-mm-dd", fg='red', nl=True)
        return False
    return validate_symbol(symbol)

def print_help_msg(command):
    with click.Context(command) as ctx:
        click.echo(command.get_help(ctx))

def validate_symbol(symbol):
    if not symbol:
        click.secho("Please provide security/index code", fg='red', nl=True)
        return False
    return True




from nseta.common.history import get_history
from nseta.plots.plots import *
from nseta.cli.inputs import *

import click
from datetime import datetime

PLOT_KEY_TO_FUNC = {"ALL": plot_technical_indicators,
			   "PRICE": plot_history,
               "RSI": plot_rsi,
               "EMA": plot_ema,
               "SMA": plot_sma,
               "SSTO": plot_sstochastic,
               "FSTO": plot_fstochastic,
               "ADX": plot_adx,
               "MACD": plot_macd,
               "MOM": plot_mom,
               "DMI": plot_dmi,
               "BBANDS": plot_bbands}
PLOT_TI_KEYS = list(PLOT_KEY_TO_FUNC.keys())

@click.command(help='Plot various technical analysis indicators')
@click.option('--symbol', '-S',  help='Security code')
@click.option('--start', '-s', help='Start date in yyyy-mm-dd format')
@click.option('--end', '-e', help='End date in yyyy-mm-dd format')
@click.option('--plot-type', '-p','plot_type', default='ALL', help=', '.join(PLOT_TI_KEYS) + ". Choose one.")
def plot_ta(symbol, start, end, plot_type="ALL"):
	if not validate_inputs(start, end, symbol):
		print_help_msg(plot_ta)
		return
	sd = datetime.strptime(start, "%Y-%m-%d").date()
	ed = datetime.strptime(end, "%Y-%m-%d").date()

	df = get_history(symbol, sd, ed)
	df.set_index('Date', inplace=True)
	plot_type = plot_type.upper()
	if plot_type in PLOT_KEY_TO_FUNC:
		PLOT_KEY_TO_FUNC[plot_type](df).show()
	else:
		PLOT_KEY_TO_FUNC['ALL'](df).show()



# -*- coding: utf-8 -*-
"""
Spyder Editor

This is a temporary script file.
"""
import setuptools  # noqa
from distutils.core import setup

with open("README.md", "r") as fh:
    long_description = fh.read()
with open("requirements.txt", "r") as fh:
    install_requires = fh.read().splitlines()

setup(
  name = 'nseta',
  packages=setuptools.find_packages(where="./", exclude=["docs", "tests"]),
  version = '0.1',
  description = 'Library to analyse and predict financial data from National Stock Exchange (NSE - India) in pandas dataframe ',
  long_description = long_description,
  long_description_content_type="text/markdown",
  author = 'Praveen K Jha',
  author_email = 'pkjmesra@gmail.com',
  url = 'https://github.com/pkjmesra/nseta', # use the URL to the github repo
  entry_points='''
    [console_scripts]
    nsetacli=nseta.cli.nsetacli:nsetacli
  ''',
  download_url = 'https://github.com/pkjmesra/nseta/archive/v0.1.zip', 
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    ],
  install_requires = install_requires,
  keywords = ['NSE', 'Technical Indicators', 'Backtesting', 'Forecasting'],
),
python_requires='>=3.6',



import click
from nseta.common.history import get_history, get_index_pe_history
from nseta.cli.inputs import *
from datetime import datetime


@click.command(help='Get price history of a security for given dates')
@click.option('--symbol', '-S',  help='Security code')
@click.option('--start', '-s', help='Start date in yyyy-mm-dd format')
@click.option('--end', '-e', help='End date in yyyy-mm-dd format')
@click.option('--series', default='EQ', help='Default series - EQ (Equity)')
@click.option('--file', '-o', 'file_name',  help='Output file name')
@click.option('--index/--no-index', default=False, help='--index if security is index else --no-index')
@click.option('--format', '-f', default='csv',  type=click.Choice(['csv', 'pkl']),
                help='Output format, pkl - to save as Pickel and csv - to save as csv')
def history(symbol, start, end, series, file_name, index, format): #, futures, expiry, option_type, strike):
    if not validate_inputs(start, end, symbol):
        print_help_msg(history)
        return
    sd = datetime.strptime(start, "%Y-%m-%d").date()
    ed = datetime.strptime(end, "%Y-%m-%d").date()
    df = get_history(symbol, sd, ed)
    click.echo(df.head())
    if not file_name:
        file_name = symbol + '.' + format
    if format == 'csv':
        df.to_csv(file_name)
    else:
        df.to_pickle(file_name)
    click.secho('Saved to: {}'.format(file_name), fg='green', nl=True)

@click.command(help='Get PE history of a security for given dates')
@click.option('--symbol', '-S',  help='Index code')
@click.option('--start', '-s', help='Start date in yyyy-mm-dd format')
@click.option('--end', '-e', help='End date in yyyy-mm-dd format')
@click.option('--format', '-f', default='csv',  type=click.Choice(['csv', 'pkl']),
                help='Output format, pkl - to save as Pickel and csv - to save as csv')
@click.option('--file', '-o', 'file_name',  help='Output file name')
def pe_history(symbol, start, end, format, file_name):
    if not validate_inputs(start, end, symbol):
        print_help_msg(pe_history)
        return
    df = get_index_pe_history(symbol, sd, ed)
    click.echo(df.head())
    
    if not file_name:
        file_name = symbol + '.' + format

    if format == 'csv':
        df.to_csv(file_name)
    else:
        df.to_pickle(file_name)
    click.secho('Saved to: {}'.format(file_name) , fg='green', nl=True)



from nseta.strategy.strategy import *
from nseta.common.history import *
from .inputs import *

import click
from datetime import datetime

def smac_strategy(df, autosearch):
	if not autosearch:
		backtest_smac_strategy(df, fast_period=10, slow_period=50)
	else:
		backtest_smac_strategy(df, fast_period=range(10, 30, 3), slow_period=range(40, 60, 3))

def emac_strategy(df, autosearch):
	if not autosearch:
		backtest_smac_strategy(df, fast_period=10, slow_period=50)
	else:
		backtest_smac_strategy(df, fast_period=range(10, 30, 3), slow_period=range(40, 50, 3))

def bbands_strategy(df, autosearch):
	backtest_bbands_strategy(df, period=20, devfactor=2.0)

def rsi_strategy(df, autosearch):
	if not autosearch:
		backtest_rsi_strategy(df, rsi_period=14)
	else:
		backtest_rsi_strategy(df, rsi_period=[5,7,11,14], rsi_lower=[15,30])

def macd_strategy(df, autosearch):
	if not autosearch:
		backtest_macd_strategy(df, fast_period=12, slow_period=26)
	else:
		backtest_macd_strategy(df, fast_period=range(4, 12, 2), slow_period=range(14, 26, 2))

def multi_strategy(df, autosearch):
	if not autosearch:
		backtest_multi_strategy(df, key_variable= "smac", fast_period=10, slow_period=50)
	else:
		# key_variables = ["smac", "emac", "macd"]
		result_smac = backtest_multi_strategy(df, key_variable= "smac", fast_period=10, slow_period=[40,50], rsi_lower=[15,30], rsi_upper=70)
		result_emac = backtest_multi_strategy(df, key_variable= "emac", fast_period=10, slow_period=[40,50], rsi_lower=[15,30], rsi_upper=70)
		result_macd = backtest_multi_strategy(df, key_variable= "macd", fast_period=12, slow_period=[26,40], rsi_lower=[15,30], rsi_upper=70)
		print(result_smac[['smac.fast_period', 'smac.slow_period', 'rsi.rsi_lower', 'rsi.rsi_upper', 'init_cash', 'final_value', 'pnl']].head())
		print ('\n')
		print(result_emac[['emac.fast_period', 'emac.slow_period', 'rsi.rsi_lower', 'rsi.rsi_upper', 'init_cash', 'final_value', 'pnl']].head())
		print('\n')
		print(result_macd[['macd.fast_period', 'macd.slow_period', 'rsi.rsi_lower', 'rsi.rsi_upper', 'init_cash', 'final_value', 'pnl']].head())
		print('\n')

STRATEGY_MAPPING = {
    "rsi": rsi_strategy,
    "smac": smac_strategy,
    # "base": BaseStrategy,
    "macd": macd_strategy,
    "emac": emac_strategy,
    "bbands": bbands_strategy,
    # "buynhold": BuyAndHoldStrategy,
    # "sentiment": SentimentStrategy,
    # "custom": CustomStrategy,
    # "ternary": TernaryStrategy,
    "multi": multi_strategy
}

KEY_MAPPING = {
    'dt': 'Date',
    'open': 'Open',
    'high': 'High',
    'low': 'Low',
    'close': 'Close',
    'volume': 'Volume',
}

STRATEGY_MAPPING_KEYS = list(STRATEGY_MAPPING.keys()) + ['custom']

@click.command(help='Measure the performance of your trading strategy')
@click.option('--symbol', '-S',  help='Security code')
@click.option('--start', '-s', help='Start date in yyyy-mm-dd format')
@click.option('--end', '-e', help='End date in yyyy-mm-dd format')
@click.option('--strategy', default='rsi', type=click.Choice(STRATEGY_MAPPING_KEYS), 
	help=', '.join(STRATEGY_MAPPING_KEYS) + ". Choose one.")
@click.option('--upper', '-u', default=1.5, help='Only when strategy is "custom". We buy the security when the predicted next day return is > +{upper} %')
@click.option('--lower', '-l', default=1.5, help='Only when strategy is "custom". We sell the security when the predicted next day return is < -{lower} %')
@click.option('--autosearch/--no-autosearch', default=False, 
	help='--auto for allowing to automatically measure the performance of your trading strategy on multiple combinations of parameters.')
def test_trading_strategy(symbol, start, end, autosearch, strategy, upper, lower):
	if not validate_inputs(start, end, symbol):
		print_help_msg(test_trading_strategy)
		return
	sd = datetime.strptime(start, "%Y-%m-%d").date()
	ed = datetime.strptime(end, "%Y-%m-%d").date()

	df = get_history(symbol, sd, ed)
	df['datetime'] = df['Date']
	strategy = strategy.lower()
	if strategy in STRATEGY_MAPPING:
		STRATEGY_MAPPING[strategy](df, autosearch)
	elif strategy == 'custom':
		for key in KEY_MAPPING.keys():
			df[key] = df[KEY_MAPPING[key]]
		backtest_custom_strategy(df, strategy, upper, lower)
	else:
		STRATEGY_MAPPING['rsi'](df, autosearch)

@click.command(help='Forecast & measure performance of a trading model')
@click.option('--symbol', '-S',  help='Security code')
@click.option('--start', '-s', help='Start date in yyyy-mm-dd format')
@click.option('--end', '-e', help='End date in yyyy-mm-dd format')
@click.option('--strategy', default='rsi', type=click.Choice(STRATEGY_MAPPING_KEYS), 
	help=', '.join(STRATEGY_MAPPING_KEYS) + ". Choose one.")
@click.option('--upper', '-u', default=1.5, help='Only when strategy is "custom". We buy the security when the predicted next day return is > +{upper} %')
@click.option('--lower', '-l', default=1.5, help='Only when strategy is "custom". We sell the security when the predicted next day return is < -{lower} %')
def forecast_strategy(symbol, start, end, strategy, upper, lower):
	if not validate_inputs(start, end, symbol):
		print_help_msg(forecast_strategy)
		return
	sd = datetime.strptime(start, "%Y-%m-%d").date()
	ed = datetime.strptime(end, "%Y-%m-%d").date()

	df = get_history(symbol, sd, ed)
	for key in KEY_MAPPING.keys():
		df[key] = df[KEY_MAPPING[key]]
	df.set_index('dt', inplace=True)
	df.drop(EQUITY_HEADERS, axis = 1, inplace = True)
	plt, result = daily_forecast(df, symbol, strategy, upper_limit=float(upper), lower_limit=float(lower), periods=28)
	if plt is not None:
		plt.show()



import click
from nseta.cli.historycli import *
from nseta.cli.modelcli import *
from nseta.cli.plotscli import *
from nseta.cli.strategycli import *
from nseta.cli.livecli import *

@click.group()
@click.option('--debug/--no-debug', default=False, help='--debug to turn debugging on. Default is off')
def nsetacli(debug):
    if debug:
        click.echo('Debug mode is %s' % ('on' if debug else 'off'))

nsetacli.add_command(history)
nsetacli.add_command(pe_history)
nsetacli.add_command(plot_ta)
nsetacli.add_command(create_cdl_model)
nsetacli.add_command(test_trading_strategy)
nsetacli.add_command(forecast_strategy)
nsetacli.add_command(live_quote)

if __name__ == '__main__':
    nsetacli()
